/************************************************************************/
/*									*/
/*			Access Dallas 1-Wire Devices			*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/
#include "main.h"


#ifndef W1_PIN
#define W1_PIN  PB1
#define W1_IN   PINB
#define W1_OUT  PORTB
#define W1_DDR  DDRB
#endif


bit w1_reset(void)
{
  bit err;				// uchar err
  W1_OUT &= ~(1<<W1_PIN);		// W1_PIN =0	
  W1_DDR |= 1<<W1_PIN;			// W1_DDR =1
  DELAY( DELAY_US(500));		// 480 µs
  cli();				// interrupt deaktiviert
  W1_DDR &= ~(1<<W1_PIN);		// W1_DDR =0
  DELAY( DELAY_US( 30 ));		// 66 µs 30µs
  err = W1_IN & (1<<W1_PIN);		// err=( PIN & (1<<BIT) )	//no presence detect
  sei();				// interrupt aktiviert
  DELAY( DELAY_US( 480-70 ));		// 480-66 us
  if( (W1_IN & (1<<W1_PIN)) == 0 )      // wenn W1_IN ist 0 		//no sensor
  err = 1;				// err = 1
  return err;
}


uchar w1_bit_io( bit b )
{
  cli();				// interrupt deaktiviert
  W1_DDR |= 1<<W1_PIN;			// WI_DDR =1
  DELAY( DELAY_US( 2 ));		// 1 µs
  if( b )				// wenn b dann
    W1_DDR &= ~(1<<W1_PIN);		// W1_DDR =0
  DELAY( DELAY_US( 15 - 3 ));		// 15-1 µs
  if( (W1_IN & (1<<W1_PIN)) == 0 )	// wenn W1_IN ist 0
    b = 0;				// b =0
  DELAY( DELAY_US( 60 - 15 ));		// 60-15 µs
  W1_DDR &= ~(1<<W1_PIN);		// W1_DDR =0
  sei();				// interrupt aktiviert
  return b;				// b zurückgeben
}




uint w1_byte_wr( uchar b )
{
  uchar i = 8, j;			// vaeable i und J definieren i=8 
  do{					// tu
    j = w1_bit_io( b & 1 );		// j ist der ausgabewert von w1_bit_io mit der eingabe b und 1
    b >>= 1;				// nächstes bit von b
    if( j )				// wenn j
      b |= 0x80;			// b = b | 0x80 b ist b oder 0x80
  }while( --i );			// bis i-- nicht mehr geht
  return b;
}


uint w1_byte_rd( void )
{
return w1_byte_wr( 0xff );
}


uchar w1_rom_search( uchar diff, uchar idata *id )
{
  uchar i, j, next_diff;
  bit b;

  if( w1_reset() )
    return PRESENCE_ERR;			// error, no device found
  w1_byte_wr( SEARCH_ROM );			// ROM search command
  next_diff = LAST_DEVICE;			// unchanged on last device
  i = 8 * 8;					// 8 bytes
  do{
    j = 8;					// 8 bits
    do{
      b = w1_bit_io( 1 );			// read bit
      if( w1_bit_io( 1 ) ){			// read complement bit
	if( b )					// 11
	  return DATA_ERR;			// data error
      }else{
	if( !b ){				// 00 = 2 devices
	  if( diff > i ||
	    ((*id & 1) && diff != i) ){
	    b = 1;				// now 1
	    next_diff = i;			// next pass 0
	  }
	}
      }
      w1_bit_io( b );     			// write bit
      *id >>= 1;
      if( b )					// store bit
	*id |= 0x80;
      i--;
    }while( --j );
    id++;					// next byte
  }while( i );
  return next_diff;				// to continue search
}


void w1_command( uchar command, uchar idata *id )
{
  uchar i;
  w1_reset();
  if( id ){
    w1_byte_wr( MATCH_ROM );			// to a single device
    i = 8;
    do{
      w1_byte_wr( *id );
      id++;
    }while( --i );
  }else{
    w1_byte_wr( SKIP_ROM );			// to all devices
  }
  w1_byte_wr( command );
}
